--rawset(_G, "WARE_MINIGAMES", {})
local WARE_MINIGAMES = {}
local WARE_REFERENCE = {}
local INAMEMAXLEN = 12
local BADMINIS = 0

/*
Blacklist: Prevents some minigames from being raffled. Normal conditions still cast.
Whitelist: The inverse, only rolls chosen minigames. Conditions still apply.
If this results in an empty table at the moment of raffling, list is ignored.
*/

WAREPERM.bl = {}
local bl = WAREPERM.bl
bl.black = true
bl.on = false
bl.list = {}
local function rel()
	return bl.black and "blacklist" or "whitelist"
end

// table.sort(The.boss)
local bl_cast = {
	on = function(p)
		CONS_Printf(p, "The "..rel().." has been enabled.")
		bl.on = true
	end,
	off = function(p)
		CONS_Printf(p, "The "..rel().." has been disabled.")
		bl.on = false
	end,
	modal = function(p,args)
		if #args == 0 then
			CONS_Printf(p, "This functionality acts like a "..rel()..".")
			CONS_Printf(p, "Change this with \130ware_blacklist modal <w|b>\128.")
		else
			local the = args[1]
			if the == "w" then bl.black = false
			elseif the == "b" then bl.black = true
			else CONS_Printf(p, "Unknown argument #2!");return end
			CONS_Printf(p, "This list is now a "..rel().."!")
		end
	end,
	add = function(p,args)
		if #args == 0 then
			CONS_Printf(p, "Usage: ware_blacklist add <minigame>")
			return
		end
		local iname = args[1]
		if not WARE_REFERENCE[iname] then
			CONS_Printf(p, "There is no minigame under that name!")
			return
		end
		if findValueInTable(bl.list, iname) then
			CONS_Printf(p, "That minigame is already on the list!")
			return
		end
		table.insert(bl.list, iname)
		table.sort(bl.list)
		CONS_Printf(p, "Minigame added to the "..rel()..".")
	end,
	remove = function(p,args)
		if #args == 0 then
			CONS_Printf(p, "Usage: ware_blacklist remove <minigame>")
			return
		end
		local iname = args[1]
		if not WARE_REFERENCE[iname] then
			CONS_Printf(p, "There is no minigame under that name!")
			return
		end
		if not findValueInTable(bl.list, iname) then
			CONS_Printf(p, "That minigame is not on the list!")
			return
		end
		table.remove(bl.list, findValueInTable(bl.list, iname))
		--table.sort(bl) -- No sorting needed
		CONS_Printf(p, "Minigame removed to the "..rel()..".")
	end,
	wipe = function(p,args)
		if #args == 0 then
			CONS_Printf(p, "Usage: ware_blacklist wipe <*>")
			CONS_Printf(p, "This will wipe the "..rel().."! Enter anything as second argument to confirm.")
			return
		end
		for i = #bl.list,1 do bl.list[i] = nil end
		CONS_Printf(p, "Wiped!")
	end,
	
	
	show = function(p)
		CONS_Printf(p, "The "..rel().." is "..(bl.on and "enabled" or "disabled")..".")
		if #bl.list == 0 then CONS_Printf(p, "The list is empty.");return end
		CONS_Printf(p, "The "..rel().." contains the following:")
		for i = 1,#bl.list do
			local n = bl.list[i]
			CONS_Printf(p, "\130"..n)
		end
	end,
}
COM_AddCommand("ware_blacklist", function(p,...)
	local args = {...}
	local f = table.remove(args, 1)
	if bl_cast[f] ~= nil then
		if f == "show" or (IsPlayerAdmin(p) or p == server) then
			bl_cast[f](p,args)
		else
			CONS_Printf(p, "Only administrators can use this!")
		end
	else
		CONS_Printf(p, [[ware_blacklist <option> <...>
Option can be:
show - Shows all minigames on the list, as well as modal. (Usable by everyone)
on - Toggles the feature on.
off - Toggles the feature off.
modal <w|b> - Toggles between a whitelist or a blacklist.
add <name> - Adds a minigame to the list.
remove <name> - Removes a minigame from the list.
wipe - Clears the list!

This command is only available for CustomWare or below.
]])
	end
end)
--rawset(_G, "WARE_BlacklistStatus", function() return bl.black,bl.on end)

COM_AddCommand("ware_list", function(p,...)
	local args = {...}
	if findValueInTable(args, "-plain") then
		for i = 1,#WARE_MINIGAMES do
			local mini = WARE_MINIGAMES[i]
			local str = mini.iname
			if mini.boss then str = "boss "..$ end
			print(str)
		end
	else
		CONS_Printf(p, "There are "..#WARE_MINIGAMES.." minigames total.\n")
		local The = {boss={},mini={}}
		for i = 1,#WARE_MINIGAMES do
			local mini = WARE_MINIGAMES[i]
			local tgt = The.mini
			if mini.boss then tgt = The.boss end
			table.insert(tgt, mini.iname)
		end
		local gaming = function(a,b)
			if a[3] == b[3] then
				return a[2] < b[2]
			else return a[3] < b[3] end
		end
		table.sort(The.boss)
		table.sort(The.mini)
		
		local pront = function(t)
			local ElEstrin = ""
			for ix = 1,#t do
				if ix > 1 and (ix-1)%3 == 0 then ElEstrin = $.."\n" end
				local iname = t[ix]
				local mini = WARE_MINIGAMES[WARE_REFERENCE[t[ix]]]
				local prefix = ""
				if mini.incomplete then prefix = "\143"
				elseif (mini.condition and mini.condition()) or (not mini.condition) then
					--prefix = "\136"
					prefix = "\138"
				else prefix = "\134" end
				ElEstrin = $..prefix..((" "):rep(INAMEMAXLEN-iname:len()))..iname.."\128 | "
			end
			print(ElEstrin)
		end
		
		CONS_Printf(p, #The.mini.." of them are normal minigames:")
		pront(The.mini)
		CONS_Printf(p, "\n"..#The.boss.." of them are boss minigames:")
		pront(The.boss)
		CONS_Printf(p, "")
		
		if BADMINIS > 0 then
			--CONS_Printf(p, "Some ("..BADMINIS..") are not in the list. (indev)")
		end
	end
end, 0)

rawset(_G, "WARE_AddMinigame", function(is, t)
	if t.DOESNOTCOUNT then print(is.." DOES NOT COUNT");BADMINIS = $+1;return end
	// Adds a new minigame!
	// The internal name is only used for forcing purposes...
	t.iname = is
	table.insert(WARE_MINIGAMES, t)
	
	print("Added minigame "..is)
	WARE_REFERENCE[is] = #WARE_MINIGAMES
	WARE_REFERENCE[#WARE_MINIGAMES] = is
	// yeah boye
end)

rawset(_G, "WARE_FindMinigame", function(what)
	// Given a string, returns the minigame index
	// Or the other way around
	return WARE_REFERENCE[what]
end)
rawset(_G, "WARE_GetMinigame", function(n)
	// Fuck you
	return WARE_MINIGAMES[n]
end)
rawset(_G, "WARE_FindGetMinigame", function(what)
	// pfft
	return WARE_MINIGAMES[WARE_REFERENCE[what]]
end)

rawset(_G, "WARE_FetchMinigames", function()
	// Returns a random minigame
end)
rawset(_G, "WARE_Raffle", function(ignoreBL)
	// Returns a random minigame
	// let the games begin!
	--print(WCV.force)
	if gametype >= GT_WARECUSTOM then
		if WARE_FindMinigame(WCV.force.string) ~= nil then
			print("Minigame has been forced")
			return WCV.force.string,WARE_FindMinigame(WCV.force.string)
		end
		if bl.on and #bl.list == 0 then
			print("The "..rel().." is enabled, but the list is empty!")
			ignoreBL = true
		end
	end
	if gametype == GT_WARE then ignoreBL = true end // Always ignore
	
	--local the = randomChoose(WARE_MINIGAMES)
	--return the.iname,the
	
	local minis = {}
	for i = 1,#WARE_MINIGAMES do
		if WARE_MINIGAMES[i] then // Why wouldn't it be
			local mini = WARE_MINIGAMES[i]
			if mini.incomplete then continue end // Never play incomplete ones
			if mini.condition ~= nil and mini.condition() == false then continue end // Condition hasn't passed
			if not ignoreBL and gametype >= GT_WARECUSTOM and bl.on then
				local found = findValueInTable(bl.list, mini.iname)
				if found and bl.black then continue // Blacklisted
				elseif not found and not bl.black then continue end // Not on whitelist
			end
			if WARE.boss then
				if mini.boss then minis[#minis+1] = i end
			else
				if not mini.boss then minis[#minis+1] = i end
			end
		end
	end
	
	if #minis == 0 then
		if bl.on and not ignoreBL then
			print("Unable to raffle a minigame! Trying again without the "..rel().."...")
			return WARE_Raffle(true)
		end
		error("Unable to raffle a minigame!?",2)
	end
	
	local the = randomChoose(minis)
	return WARE_MINIGAMES[the].iname,WARE_MINIGAMES[the]
	
end)

/*
the WARE global is read+write - Avoid writing to WARE and write to WARE.mem instead
WARE.mem is cleared after every state change

minigame = {
	prompt = * -- Shows a prompt for the first 8 (scaled) seconds
	-- Can be a single string or a table with multiple strings
	-- The prompt table is then passed to player.waremem.prompt
	
	DOESNOTCOUNT = boolean -- If this is true, this minigame is forcefully not added by WARE_AddMinigame
	-- For indev purposes
	incomplete = boolean -- Whether this minigame is incomplete - Exclude it from raffle
	condition = argumentless function -- Condition for this to be raffled - return true for yes
	
	music = string -- Music to play during the minigame, can be nil
	musicloop = boolean -- If this loops
	notemposhift = boolean -- Whether temposhift can interact with this minigame, true by default
	timelimit = tics -- Amount of time for minigame to last, will autoscale
	weaponrings = boolean -- Whether to show the weapon ring bar, default false
	
	skin = string -- Skin to force players into, can be nil for no forcing
	-- You can force skins during fStart with WARE_ForcePlayerSkin(player, skin, color/nil)
	
	skynum = integer -- Guess
	spawnpoint = table -- 3D vector table for where to put players when minigame starts
	positioning = table -- How to drop players: next to spawnpoint, scattered, how?
	-- ex: {"radius",160*FRACUNIT} or {"scatter",{x=0,y=0,z=0},{x=512,y=512,z=512}}
	-- If you can avoid using this that would be better
	restoreonend = boolean -- Whether to return players back to main area on minigame end, implicitly true by spawnpoint existing
	
	nodeath = boolean -- Whether receiving death damage instead results in entering pain frames, false
	forcedeath = boolean -- Whether receiving any damage results in dying, false
	loseondeath = boolean -- Whether dead players automatically lose, by default true
	
	fStart = function() -- Function to execute on minigame start, after the minigame has been prepped
	
	fThink = function() -- Function to run every frame
	fHud = function(v,p,c) -- Function to run on the game hud of each player
	
	fEnd = function() -- Function that runs when the minigame is over and after any other thinker here
	
	hShouldDamage
	hMobjCollide
	hThis
	hThat -- Function, formats as you'd expect
}
*/

rawset(_G, "MODULBASE", true)